package be.iminds.ilabt.jfed.experimenter_gui.tasks;

import be.iminds.ilabt.jfed.experimenter_gui.editor.EditableRspec;
import be.iminds.ilabt.jfed.experimenter_gui.editor.ExperimentEditorTab;
import be.iminds.ilabt.jfed.experimenter_gui.editor.ExperimentEditorTabFactory;
import be.iminds.ilabt.jfed.experimenter_gui.editor.rspec_validation.RspecCheckReason;
import be.iminds.ilabt.jfed.experimenter_gui.editor.rspec_validation.RspecValidationEngine;
import be.iminds.ilabt.jfed.experimenter_gui.editor.rspec_validation.RspecValidationResult;
import be.iminds.ilabt.jfed.experimenter_gui.ui.ExperimentEditorTabHost;
import be.iminds.ilabt.jfed.rspec.model.ModelRspec;
import be.iminds.ilabt.jfed.rspec.model.ModelRspecType;
import be.iminds.ilabt.jfed.rspec.parser.RspecParseException;
import be.iminds.ilabt.jfed.rspec.rspec_source.RequestRspecSource;
import be.iminds.ilabt.jfed.ui.javafx.dialogs.JFDialogs;
import javafx.application.Platform;
import javafx.concurrent.Task;
import javafx.stage.Window;

import java.util.concurrent.FutureTask;
import java.util.concurrent.RunnableFuture;

public class OpenRequestRspecFromStringTask extends Task<EditableRspec> {
    private final Window parentWindow;

    private final String requestRspec;

    private final ExperimentEditorTabFactory experimentEditorTabFactory;
    private final ExperimentEditorTabHost experimentEditorTabHost;

    private final RspecValidationEngine validationEngine = new RspecValidationEngine();

    OpenRequestRspecFromStringTask(Window parentWindow, String requestRspec, ExperimentEditorTabFactory experimentEditorTabFactory, ExperimentEditorTabHost experimentEditorTabHost) {
        this.parentWindow = parentWindow;
        this.requestRspec = requestRspec;
        this.experimentEditorTabFactory = experimentEditorTabFactory;
        this.experimentEditorTabHost = experimentEditorTabHost;

        updateTitle("Parsing request Rspec");
    }

    @Override
    protected EditableRspec call() throws Exception {
        final RequestRspecSource requestRspecSource =
                new RequestRspecSource(requestRspec, ModelRspecType.FX);

        RunnableFuture<RspecValidationResult> verifyCall =
                new FutureTask<>(() -> validationEngine.validateRSpec(RspecCheckReason.OPENED_FILE, requestRspecSource, parentWindow));
        Platform.runLater(verifyCall);

        RequestRspecSource resultRspecSource;

        RspecValidationResult verifyRspecResult = verifyCall.get();
        if (verifyRspecResult != null) {
            //users fixed something automatically, so we replace the source
            resultRspecSource = verifyRspecResult.getReplacementRequestRspecSource();
        } else {
            //if no replacement is done, there are no errors, or the user has been informed of them
            //either way, we put the file content in the editor, however wrong it may be
            resultRspecSource = requestRspecSource;
        }

        /*
        if the ModelRspec can be made without losing data, we modify our ResultRspecSource so that this ModelRspec
        is the primary source. This will make sure that the users sees the loaded rspec first in the topology editor.
         */
        if (!resultRspecSource.isLosingData(ModelRspecType.FX)) {
            ModelRspec modelRspec = resultRspecSource.getModelRspec(ModelRspecType.FX);
            if (modelRspec == null)
                throw new IllegalStateException("jFed bug: FX modelrspec is not losing data, yet it is null (should never happen)");
            resultRspecSource = new RequestRspecSource(modelRspec);
        }

        return new EditableRspec(resultRspecSource, "request.rspec", null);
    }

    @Override
    protected void succeeded() {
        ExperimentEditorTab newTab =
                experimentEditorTabFactory.createExperimentEditorTab(getValue());
        experimentEditorTabHost.addExperimentEditorTab(newTab);
        experimentEditorTabHost.showExperimentEditorTab(newTab);
    }

    @SuppressWarnings("ThrowableResultOfMethodCallIgnored")
    @Override
    protected void failed() {
        if (getException() instanceof RspecParseException) {
            RspecParseException ex = (RspecParseException) getException();
            JFDialogs.create()
                    .owner(parentWindow)
                    .message("The specified source does not contain a valid Rspec.\n\n" +
                            (ex.getStaxLocation() == null ?
                                    "" :
                                    ("Error on line " + ex.getStaxLocation().getLineNumber() + ", column " + ex.getStaxLocation().getColumnNumber())
                            )
                            + ": " + ex.getMessage())
                    .masthead("Opening Rspec failed")
                    .title("Opening Rspec failed")
                    .showException(ex);
        } else {
            JFDialogs.create()
                    .owner(parentWindow)
                    .message("Could not open the RSpec from the given source: " + getException().getMessage())
                    .masthead("Error while opening Experiment Definition")
                    .title("Error while opening Experiment Definition")
                    .showException(getException());
        }
    }
}
